@@ -1,5 +1,5 @@
 // -*- c++ -*-
-/* $Id: dispatcher.cc,v 1.7.2.1 2005/01/21 20:12:50 murrayc Exp $ */
+/* $Id: dispatcher.cc,v 1.11.2.1 2006/05/10 13:06:42 murrayc Exp $ */
 
 /* Copyright 2002 The gtkmm Development Team
  *
@@ -66,7 +66,7 @@ struct DispatchNotifyData
     : tag (tag_), dispatcher (dispatcher_), notifier (notifier_) {}
 };
 
-void warn_failed_pipe_io(const char* what, int err_no)
+static void warn_failed_pipe_io(const char* what, int err_no)
 {
 #ifdef G_OS_WIN32
   const char *const message = g_win32_error_message(err_no);
@@ -81,7 +81,7 @@ void warn_failed_pipe_io(const char* what, int err_no)
  * Try to set the close-on-exec flag of the file descriptor,
  * so that it won't be leaked if a new process is spawned.
  */
-void fd_set_close_on_exec(int fd)
+static void fd_set_close_on_exec(int fd)
 {
   const int flags = fcntl(fd, F_GETFD, 0);
   g_return_if_fail(flags >= 0);
@@ -94,7 +94,7 @@ void fd_set_close_on_exec(int fd)
  * One word: paranoia.
  */
 #ifdef G_OS_WIN32
-void fd_close_and_invalidate(HANDLE& fd)
+static void fd_close_and_invalidate(HANDLE& fd)
 {
   if(fd != 0)
   {
@@ -105,7 +105,7 @@ void fd_close_and_invalidate(HANDLE& fd)
   }
 }
 #else /* !G_OS_WIN32 */
-void fd_close_and_invalidate(int& fd)
+static void fd_close_and_invalidate(int& fd)
 {
   if(fd >= 0)
   {
@@ -188,8 +188,10 @@ DispatchNotifier::DispatchNotifier(const Glib::RefPtr<MainContext>& context)
 {
   create_pipe();
 
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
 #ifdef G_OS_WIN32
     conn_io_handler_ = context_->signal_io().connect(
         sigc::mem_fun(*this, &DispatchNotifier::pipe_io_handler),
@@ -199,6 +201,7 @@ DispatchNotifier::DispatchNotifier(const Glib::RefPtr<MainContext>& context)
         sigc::mem_fun(*this, &DispatchNotifier::pipe_io_handler),
         fd_receiver_, Glib::IO_IN);
 #endif /* !G_OS_WIN32 */
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   }
   catch(...)
   {
@@ -209,6 +212,7 @@ DispatchNotifier::DispatchNotifier(const Glib::RefPtr<MainContext>& context)
 
     throw;
   }
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
 }
 
 DispatchNotifier::~DispatchNotifier()
@@ -231,20 +235,29 @@ void DispatchNotifier::create_pipe()
 
   if(!fd_receiver_)
   {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     GError* const error = g_error_new(G_FILE_ERROR, G_FILE_ERROR_FAILED,
                                       "Failed to create event for inter-thread communication: %s",
                                       g_win32_error_message(GetLastError()));
     throw Glib::FileError(error);
+#else 
+    return; //TODO: Provide an alternative to the exception.
+#endif //GLIBMM_EXCEPTIONS_ENABLED
   }
+
 #else /* !G_OS_WIN32 */
   int filedes[2] = { -1, -1 };
 
   if(pipe(filedes) < 0)
   {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     GError* const error = g_error_new(G_FILE_ERROR, g_file_error_from_errno(errno),
                                       "Failed to create pipe for inter-thread communication: %s",
                                       g_strerror(errno));
     throw Glib::FileError(error);
+#else 
+    return; //TODO: Provide an alternative to the exception.
+#endif //GLIBMM_EXCEPTIONS_ENABLED
   }
 
   fd_set_close_on_exec(filedes[0]);
@@ -353,14 +366,18 @@ bool DispatchNotifier::pipe_io_handler(Glib::IOCondition)
     // Actually, we wouldn't need the try/catch block because the Glib::Source
     // C callback already does it for us.  However, we do it anyway because the
     // default return value is 'false', which is not what we want.
+    #ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
     {
+    #endif //GLIBMM_EXCEPTIONS_ENABLED
       data.dispatcher->signal_();
+    #ifdef GLIBMM_EXCEPTIONS_ENABLED
     }
     catch(...)
     {
       Glib::exception_handlers_invoke();
     }
+    #endif //GLIBMM_EXCEPTIONS_ENABLED
   }
 #else /* !G_OS_WIN32 */
   DispatchNotifyData data;
@@ -390,14 +407,18 @@ bool DispatchNotifier::pipe_io_handler(Glib::IOCondition)
   // Actually, we wouldn't need the try/catch block because the Glib::Source
   // C callback already does it for us.  However, we do it anyway because the
   // default return value is 'false', which is not what we want.
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
     data.dispatcher->signal_(); // emit
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   }
   catch(...)
   {
     Glib::exception_handlers_invoke();
   }
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
 #endif /* !G_OS_WIN32 */
 
   return true;
